<?php

namespace Teknisa\DF\Factories;

use Doctrine\ORM\EntityManager;

class ParametrizacaoFiscalCentralizadaFactory
{
    private $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function consultarErrosRegra($cdregrafisc)
    {
        $sql = <<<SQL
        SELECT DISTINCT
           ERRO.CDIMPOSTO,
           I.NMIMPOSTO,
           ERRO.CDIMPOSTONFE,
           INFE.SGIMPOSTONFE,
           ERRO.CDSITUTRIB,
           P.DSSITUTRIB,
           ERRO.CDFISCAL,
           ERRO.VRALIQUOTA,
           ERRO.CDFISCOPER,
           ERRO.IDCADPADRAO,
           TO_CHAR(ERRO.IDERRO) as IDERRO
        FROM TABLE(CONSULTA_ERRO_REGRAFISCAL(:P_CDREGRAFISC)) ERRO
        LEFT JOIN IMPOSTO I
        ON I.CDIMPOSTO = ERRO.CDIMPOSTO
        LEFT JOIN IMPOSTONFE INFE
        ON I.CDIMPOSTONFE = INFE.CDIMPOSTONFE
        LEFT JOIN PARAMCST P
        ON P.CDSITUTRIB = ERRO.CDSITUTRIB
        ORDER BY ERRO.IDERRO
SQL;

        $params = array(
            "P_CDREGRAFISC" => $cdregrafisc
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function consultaRepetida($cdempresa,$identrsaidop,$cdtipooper,$cdregrafisc){
        
        $sql = <<<SQL
        SELECT DISTINCT
            V.CDREGRAFISC,
            V.NRVINCFISCREGRA,
            V.CDEMPRESA,
            E.NMEMPRESA,
            V.IDENTRSAIDOP,
            V.CDTIPOOPER,
            T.NMTIPOOPER,
            V.CDINSCESTA,
            I.NMREPRINSEST AS DSINSCESTA,
            V.NRORDPRIORIDADE,
            V.DTVALVINCFISC,
            V.NRORG
        FROM VINCULOFISREGRA V,
            EMPRESA E,
            TIPOOPERAC T,
            INSCRESTAD I
        WHERE 
            V.CDEMPRESA  = E.CDEMPRESA(+)
            AND V.IDENTRSAIDOP = T.IDENTRSAIDOP(+)
            AND V.CDTIPOOPER   = T.CDTIPOOPER(+)
            AND V.CDEMPRESA    = I.CDEMPRESA(+)
            AND V.CDINSCESTA   = I.CDINSCESTA(+)
            AND V.CDEMPRESA    = :P_CDEMPRESA
            AND V.IDENTRSAIDOP = :P_IDENTRSAIDOP
            AND V.CDTIPOOPER   = :P_CDTIPOOPER
            AND V.CDREGRAFISC  = :P_CDREGRAFISC
SQL;
        $params = array(
            "P_CDEMPRESA" => $cdempresa,
            "P_IDENTRSAIDOP" => $identrsaidop,
            "P_CDTIPOOPER" => $cdtipooper,
            "P_CDREGRAFISC" => $cdregrafisc
        );
        return ($this->entityManager->getConnection()->fetchAll($sql, $params));
        
    }
    
    public function getNewCodeRegra($nrorg)
    {
        $sql = <<<SQL
        SELECT (MAX(R.CDREGRAFISC) + 1) AS CODIGO FROM REGRAFISCAL R WHERE NRORG = :P_NRORG
SQL;

        $params = array(
            "P_NRORG" => $nrorg
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getImpostosGrupo($cdgrupoimpost, $nrorg) {
         $sql = <<<SQL
        SELECT CDGRUPIMPOSTO, CDIMPOSTO
          FROM ITAGRUPIMPOSTO
         WHERE CDGRUPIMPOSTO = :P_CDGRUPIMPOSTO
         AND   NRORG         = :P_NRORG
SQL;

        $params = array(
            "P_CDGRUPIMPOSTO" => $cdgrupoimpost,
            "P_NRORG" => $nrorg
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function saveAgrupImposto($cdRegraFisc, $cdGrupImposto) {
        $sql = "INSERT INTO IMPREGRAFISCAL (CDREGRAFISC, CDIMPOSTO, IDIMPOBRIGREGRA)
                SELECT
                   :P_CDREGRAFISC,
                   CDIMPOSTO,
                   'S'
                 FROM ITAGRUPIMPOSTO
                WHERE CDGRUPIMPOSTO = :P_CDGRUPIMPOSTO";
          $params = array(
            "P_CDGRUPIMPOSTO"   => $cdGrupImposto,
            "P_CDREGRAFISC"     => $cdRegraFisc
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getImpostosGrupoAntigo($cdregrafisc, $nrorg){
        $sql = "SELECT DISTINCT R.CDGRUPIMPOSTO, T.CDIMPOSTO 
                FROM REGRAFISCAL R 
                    JOIN ITAGRUPIMPOSTO T ON R.CDGRUPIMPOSTO = T.CDGRUPIMPOSTO 
                    JOIN IMPREGRAFISCAL P ON R.CDREGRAFISC = P.CDREGRAFISC AND P.CDIMPOSTO = T.CDIMPOSTO
                WHERE R.CDREGRAFISC = :P_CDREGRAFISC ORDER BY R.CDREGRAFISC";
                
        $params = array(
            "P_CDREGRAFISC" => $cdregrafisc
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function validaProdutoRepitidoRegra($cdregrafisc, $cdpaisdest, $sgestadest, $cdpaisorig, $sgestaorig, $idtipodestfisc, $idtipoorigfisc, $idutufdest, $idutuforig, $cdproduto, $cdimposto) {
        
        $sql = "SELECT
                    CDPRODUTO
                FROM
                    DADOSFISCPROD
                WHERE
                    (
                    (( ( CDPAISDEST = :P_CDPAISDEST  AND SGESTADEST = :P_SGESTADEST ) OR :P_IDUTUFDEST = 'N' )
                    
                    AND ( ( CDPAISORIG = :P_CDPAISORIG  AND SGESTAORIG = :P_SGESTAORIG ) OR :P_IDUTUFORIG = 'N' ))
                    
                    OR ( ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'E' )  AND  CDPAISORIG = :P_CDPAISORIG  AND SGESTAORIG = :P_SGESTAORIG )
                    OR ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'E' )
                    
                    OR  ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'E' )  
                    OR ( ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'E' ) AND  CDPAISDEST = :P_CDPAISDEST  AND SGESTADEST = :P_SGESTADEST )
                    
                    
                    OR ( IDTIPODESTFISC = 'E' AND :P_IDTIPODESTFISC = 'T' )
                    OR ( IDTIPOORIGFISC = 'E' AND :P_IDTIPOORIGFISC = 'T' )
                    
                    
                    OR ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'T' )
                    OR ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'T' )

                    
                    )
                    
                    AND CDREGRAFISC = :P_CDREGRAFISC
                    AND CDPRODUTO = :P_CDPRODUTO
                    AND CDIMPOSTO =:P_CDIMPOSTO";
                    
                    
                
        $params = array(
            "P_CDREGRAFISC"     => $cdregrafisc,
            "P_CDPAISDEST"      => $cdpaisdest,
            "P_SGESTADEST"      => $sgestadest,
            "P_CDPAISORIG"      => $cdpaisorig,
            "P_SGESTAORIG"      => $sgestaorig,
            "P_IDTIPODESTFISC"  => $idtipodestfisc,
            "P_IDTIPOORIGFISC"  => $idtipoorigfisc,
            "P_IDUTUFDEST"      => $idutufdest,
            "P_IDUTUFORIG"      => $idutuforig,
            "P_CDPRODUTO"       => $cdproduto,
            "P_CDIMPOSTO"       => $cdimposto
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getMaxdadosfiscprod($cdregrafisc, $cdimposto) {
        $sql = "SELECT NVL(MAX(D.NRSEQDADOSFISC), 0) + 1 AS CODIGO FROM DADOSFISCPROD D WHERE D.CDREGRAFISC = :P_CDREGRAFISC AND D.CDIMPOSTO = :P_CDIMPOSTO";
       
       $params = array(
            "P_CDREGRAFISC" => $cdregrafisc,
            "P_CDIMPOSTO"   => $cdimposto
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }


        
    public function getMaxdadosfiscncm($cdregrafisc, $cdimposto) {
        $sql = "SELECT NVL(MAX(D.NRSEQDADOSFISC), 0) + 1 AS CODIGO FROM DADOSFISCNCM D WHERE D.CDREGRAFISC = :P_CDREGRAFISC AND D.CDIMPOSTO = :P_CDIMPOSTO";
       
       $params = array(
            "P_CDREGRAFISC" => $cdregrafisc,
            "P_CDIMPOSTO"   => $cdimposto
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }
    
    //Replicação regra fiscal

    
    public function saveRegraFiscReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeInicial) {
      

        $sql = "INSERT INTO REGRAFISCAL (CDREGRAFISC, NMREGRAFISC, CDGRUPIMPOSTO, DSMENSREGRAFISC, DTVALREGRAFISC, IDATIVREGRAFISC, IDUTGRUPOFISCAL, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG)
                SELECT :P_CDREGRAFISC, NMREGRAFISC, CDGRUPIMPOSTO, DSMENSREGRAFISC, :P_DTVALREGRAFISC, IDATIVREGRAFISC, IDUTGRUPOFISCAL, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG
                 FROM REGRAFISCAL
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem,
            "P_DTVALREGRAFISC"      => $validadeInicial
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveImpostoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO IMPREGRAFISCAL (CDREGRAFISC, CDIMPOSTO, IDIMPOBRIGREGRA, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, IDORIGEMPRODIMP, NRORG)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, IDIMPOBRIGREGRA, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, IDORIGEMPRODIMP, NRORG
                 FROM IMPREGRAFISCAL
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveImpPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO DADOSFISCREGRA (CDREGRAFISC, CDIMPOSTO, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, NRSEQDADOSFISC, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, VRIMPOSTOREP)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, NRSEQDADOSFISC, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, VRIMPOSTOREP
                 FROM DADOSFISCREGRA
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveMsgImpPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGFISCREGRA (CDREGRAFISC, CDIMPOSTO, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG
                 FROM MSGFISCREGRA
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveImpNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO DADOSFISCNCM (CDREGRAFISC, CDIMPOSTO, CDCLASFISC, NRSEQDADOSFISC, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, VRRANGINI, VRRANGFIM, SGUNIDEMB, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, VRIMPOSTOREP)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, CDCLASFISC, NRSEQDADOSFISC, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, VRRANGINI, VRRANGFIM, SGUNIDEMB, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, VRIMPOSTOREP
                 FROM DADOSFISCNCM
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveMsgImpNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGFISCNCM (CDREGRAFISC, CDIMPOSTO, CDCLASFISC, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, CDCLASFISC, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG
                 FROM MSGFISCNCM
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveImpProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO DADOSFISCPROD (CDREGRAFISC, CDIMPOSTO, CDPRODUTO, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, NRSEQDADOSFISC, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, IDCENARIO, VRIMPOSTOREP)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, CDPRODUTO, CDSITUTRIB, CDFISCAL, CDPAISORIG, SGESTAORIG, CDMUNIORIG, CDPAISDEST, SGESTADEST, CDMUNIDEST, VRALIQUOTA, IDMODBCNFE, IDMODBCSTNFE, IDMOTIDESSTNFE, CDTPCRPICOSTNFE, CDNATUBCPCSTNFE, VRINIFAIXA, VRFIMFAIXA, VRPARDEDUZ, VR1PERREDBC, VRPERREDBC, VRPBCSUBTRIB, IDTRATIMPST, IDPMCBASECA, IDCSTINTEREST, IDCSTINTERNA, NRSEQDADOSFISC, CDNATUREC, CDDETALHEFISC, IDTIPODESTFISC, IDTIPOORIGFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, IDORIGEMPRODIMP, CDENQLEGALIPI, CDCEST, VRADICALIQFECP, CDBENEFFISCAL, NRORG, IDCONSMVAPMC, IDCONSCOMPBCPMC, IDCONSDIFALIQ, IDIMPODESONERA, IDCONSDESONVRNF, CDCOMPST, CDRESTST, IDCENARIO, VRIMPOSTOREP
                 FROM DADOSFISCPROD
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveMsgImpProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGFISCPROD (CDREGRAFISC, CDIMPOSTO, CDPRODUTO, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG)
                SELECT :P_CDREGRAFISC, CDIMPOSTO, CDPRODUTO, NRSEQDADOSFISC, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORG
                 FROM MSGFISCPROD
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveCfopPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO CFOPADRAOREGFIS (CDREGRAFISC, CDFISCOPER, IDCFOPINTEREST, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO,NRORG)
                SELECT :P_CDREGRAFISC, CDFISCOPER, IDCFOPINTEREST, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO,NRORG
                 FROM CFOPADRAOREGFIS
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
   
    public function saveMsgCfopPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGCFOPADRAO (CDREGRAFISC, IDCFOPINTEREST, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG)
                SELECT :P_CDREGRAFISC, IDCFOPINTEREST, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG
                 FROM MSGCFOPADRAO
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveCfopNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO CFOPNCMREGRAFIS (CDREGRAFISC, CDCLASFISC, IDCFOPINTEREST, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC,IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG)
                SELECT :P_CDREGRAFISC, CDCLASFISC, IDCFOPINTEREST, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC,IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG
                 FROM CFOPNCMREGRAFIS
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
   
    public function saveMsgCfopNCMoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGCFOPNCM (CDREGRAFISC, CDCLASFISC, IDCFOPINTEREST, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG)
                SELECT :P_CDREGRAFISC, CDCLASFISC, IDCFOPINTEREST, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG
                 FROM MSGCFOPNCM
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveCfopProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO CFOPREGRAFISCAL (CDREGRAFISC, CDPRODUTO, CDFISCOPER, IDCFOPINTEREST, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG)
                SELECT :P_CDREGRAFISC, CDPRODUTO, CDFISCOPER, IDCFOPINTEREST, IDDIFEALIQ, IDDIFALIQCUSPR, CDMODDOCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG
                 FROM CFOPREGRAFISCAL
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
   
    public function saveMsgCfopProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MSGCFOPREGFISC (CDREGRAFISC, IDCFOPINTEREST,CDPRODUTO, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG)
                SELECT :P_CDREGRAFISC, IDCFOPINTEREST,CDPRODUTO, NRSEQMSG, DSMENSAGEM, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,NRORG
                 FROM MSGCFOPREGFISC
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveGrupoFiscReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO GRUPOFISCREGRA (CDREGRAFISC, CDGRUPOFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG)
                SELECT :P_CDREGRAFISC, CDGRUPOFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG
                 FROM GRUPOFISCREGRA
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function saveVinculoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeFinal) {
      

        $sql = "INSERT INTO VINCULOFISREGRA (CDREGRAFISC, NRVINCFISCREGRA, CDEMPRESA, IDENTRSAIDOP, CDTIPOOPER, CDINSCESTA, CDGRUPOFISC, NRORDPRIORIDADE, DTVALVINCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG)
                SELECT :P_CDREGRAFISC, NRVINCFISCREGRA, CDEMPRESA, IDENTRSAIDOP, CDTIPOOPER, CDINSCESTA, CDGRUPOFISC, NRORDPRIORIDADE, :P_DTVALVINCFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG
                 FROM VINCULOFISREGRA
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem,
            "P_DTVALVINCFISC"     => $validadeFinal
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
   
    public function saveMsgReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem) {
      

        $sql = "INSERT INTO MENSAGEMREGFISC (CDREGRAFISC, NRMENSREGRAFISC, DSMENSREGRAFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG)
                SELECT :P_CDREGRAFISC, NRMENSREGRAFISC, DSMENSREGRAFISC, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRORG
                 FROM MENSAGEMREGFISC
                WHERE CDREGRAFISC = :P_CDREGRAFISCORIGEM";
        
        $params = array(
            
            "P_CDREGRAFISC"       =>  $cdregrafisc,
            "P_CDREGRAFISCORIGEM" => $cdregrafiscorigem
            
        );
       
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
}

