<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;

ini_set('soap.wsdl_cache_enabled',0);
ini_set('soap.wsdl_cache_ttl',0);

class NFSeNovaLima {
    const URL_PRODUCAO = "http://200.149.10.117:8093/nfe/snissdigitalsvc?wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;

    protected $ambiente; //1 = producao, 2 = homologacao

    // Dados para identificar o Certificado Digital
    protected $cdempresa;
    protected $cdinscmuni;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $xmlConsultarLoteRps;

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;
    protected $SenhaAcesso;
    protected $FraseSecreta;

    protected $senhaCertificadoDigital;

    // Dados Prestador
    protected $InscricaoPrestador;
    protected $InscricaoMunicipalPrestador;
    protected $RazaoSocialPrestador;
    protected $CodigoMunicipioPrestador;
    protected $MunicipioIncidencia;

    // Dados Lote RPS
    protected $IdLote;
	protected $NumeroLote;

	// Dados RPS
    protected $IdRps;
	protected $NumeroRps;
	protected $SerieRps;
	protected $TipoRps;
	protected $DataEmissao;
	protected $NaturezaOperacao;
	protected $RegimeEspecialTributacao;
	protected $OptanteSimplesNacional;
	protected $IncentivadorCultural;
	protected $StatusRps;
	protected $ValorServicos;
	protected $ValorDeducoes;
	protected $ValorPis;
	protected $ValorCofins;
	protected $ValorInss;
	protected $ValorIr;
	protected $ValorCsll;
	protected $IssRetido;
	protected $ValorIss;
	protected $ValorIssRetido;
	protected $OutrasRetencoes;
	protected $BaseCalculo;
	protected $Aliquota;
	protected $ValorLiquidoNfse;
	protected $DescontoCondicionado;
	protected $DescontoIncondicionado;
	protected $ItemListaServico;
	protected $CodigoCnae;
	protected $CodigoTributacaoMunicipio;
	protected $Discriminacao;
	protected $CodigoMunicipioTomador;
	protected $InscricaoTomador;
	protected $InscricaoMunicipalTomador;
	protected $RazaoSocialTomador;
	protected $Endereco;
	protected $Numero;
	protected $Complemento;
	protected $Bairro;
	protected $Uf;
	protected $Cep;
	protected $Telefone;
	protected $Email;
	protected $CodigoPaisCliente;
	protected $CodigoPais;

	public function __construct($connection, Environment $environment, $newCode, $xmlDir) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;
        //$this->setCaminhoXml($xmlDir);
	}

	private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

	private function gerarXmlEnviarLoteRps() {
		$this->xmlEnviarLoteRps = '
		<EnviarLoteRpsEnvio xmlns="http://www.abrasf.org.br/nfse.xsd">
			<LoteRps Id="'.$this->IdLote.'" versao="2.01">
				<NumeroLote>'.$this->NumeroLote.'</NumeroLote>
				<CpfCnpj>
				    <Cnpj>'.$this->InscricaoPrestador.'</Cnpj>
				</CpfCnpj>
				<InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
				<QuantidadeRps>'.count($this->listaRps).'</QuantidadeRps>
				<ListaRps>';

				foreach ($this->listaRps as $rps) {
                    $this->xmlEnviarLoteRps .= "\n" . $rps;
                }

                $this->xmlEnviarLoteRps .= '
                </ListaRps>
			</LoteRps>
		</EnviarLoteRpsEnvio>';
	}

	private function adicionarRpsEnviarLote() {
		$xmlRps = '
				<Rps>
					<InfDeclaracaoPrestacaoServico xmlns="http://www.abrasf.org.br/nfse.xsd" Id="'.$this->IdRps.'">
					    <Rps Id="'.$this->IdRps.'">
    						<IdentificacaoRps>
    							<Numero>'.(int) $this->NumeroRps.'</Numero>
    							<Serie>'.$this->SerieRps.'</Serie>
    							<Tipo>'.$this->TipoRps.'</Tipo>
    						</IdentificacaoRps>
    						<DataEmissao>'.substr($this->DataEmissao, 0, 10).'</DataEmissao>
    						<Status>'.$this->StatusRps.'</Status>
    					</Rps>
    					<Competencia>'.substr($this->DataEmissao, 0, 10).'</Competencia>
						<Servico>
							<Valores>
								<ValorServicos>'.Util::formatarNumero($this->ValorServicos).'</ValorServicos>
								<ValorDeducoes>'.Util::formatarNumero($this->ValorDeducoes).'</ValorDeducoes>
								<ValorPis>'.Util::formatarNumero($this->ValorPis).'</ValorPis>
								<ValorCofins>'.Util::formatarNumero($this->ValorCofins).'</ValorCofins>
								<ValorInss>'.Util::formatarNumero($this->ValorInss).'</ValorInss>
								<ValorIr>'.Util::formatarNumero($this->ValorIr).'</ValorIr>
								<ValorCsll>'.Util::formatarNumero($this->ValorCsll).'</ValorCsll>
								<OutrasRetencoes>'.Util::formatarNumero($this->OutrasRetencoes).'</OutrasRetencoes>
								<ValorIss>'.Util::formatarNumero($this->ValorIss).'</ValorIss>
								<Aliquota>'.$this->Aliquota.'</Aliquota>
								<DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
								<DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
							</Valores>
							<IssRetido>'.$this->IssRetido.'</IssRetido>
							    ';
							if($this->IssRetido == 1) {
							    $xmlRps.='
							    <ResponsavelRetencao>1</ResponsavelRetencao>';
							}
							$xmlRps.='
							<ItemListaServico>'.$this->ItemListaServico.'</ItemListaServico>
							<CodigoCnae>'.$this->CodigoTributacaoMunicipio.'</CodigoCnae>
							<CodigoTributacaoMunicipio>'.$this->CodigoTributacaoMunicipio.'</CodigoTributacaoMunicipio>
							<Discriminacao>'.$this->Discriminacao.'</Discriminacao>
							<CodigoMunicipio>'.$this->CodigoMunicipioPrestador.'</CodigoMunicipio>
							<CodigoPais>'.$this->CodigoPais.'</CodigoPais>
							<ExigibilidadeISS>1</ExigibilidadeISS>
							<MunicipioIncidencia>'.$this->MunicipioIncidencia.'</MunicipioIncidencia>
						</Servico>
						<Prestador>
						    <CpfCnpj>
						    	<Cnpj>'.$this->InscricaoPrestador.'</Cnpj>
						    </CpfCnpj>
							<InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipal>
							<Senha>'.$this->SenhaAcesso.'</Senha>
							<FraseSecreta>'.$this->FraseSecreta.'</FraseSecreta>
						</Prestador>
						<Tomador>
							<IdentificacaoTomador>
								<CpfCnpj>
									';

								if (strlen($this->InscricaoTomador) == 11) {
									$xmlRps .= '<Cpf>'.$this->InscricaoTomador.'</Cpf>';
								}
								else {
									$xmlRps .= '<Cnpj>'.$this->InscricaoTomador.'</Cnpj>';
								}

								$xmlRps .= '
								</CpfCnpj>
							<InscricaoMunicipal>'.$this->InscricaoMunicipalTomador.'</InscricaoMunicipal>
							</IdentificacaoTomador>
							<RazaoSocial>'.$this->RazaoSocialTomador.'</RazaoSocial>
							<Endereco>
								<Endereco>'.$this->Endereco.'</Endereco>
								<Numero>'.$this->Numero.'</Numero>
								<Complemento>'.$this->Complemento.'</Complemento>
								<Bairro>'.$this->Bairro.'</Bairro>
								<CodigoMunicipio>'.$this->CodigoMunicipioTomador.'</CodigoMunicipio>
								<Uf>'.$this->Uf.'</Uf>
								<CodigoPais>'.$this->CodigoPaisCliente.'</CodigoPais>
								<Cep>'.$this->Cep.'</Cep>
							</Endereco>
							<Contato>
							    <Telefone>'.$this->Telefone.'</Telefone>
							    <Email>'.$this->Email.'</Email>
							</Contato>
						</Tomador>
						<RegimeEspecialTributacao>'.$this->RegimeEspecialTributacao.'</RegimeEspecialTributacao>
						<OptanteSimplesNacional>'.$this->OptanteSimplesNacional.'</OptanteSimplesNacional>
						<IncentivoFiscal>'.$this->IncentivoFiscal.'</IncentivoFiscal>
						<Producao>'.$this->ambiente.'</Producao>
					</InfDeclaracaoPrestacaoServico>
				</Rps>';

        /*
        valores

        <ValorIssRetido>'.Util::formatarNumero($this->ValorIssRetido).'</ValorIssRetido>
        <DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
        <DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
        */

        $this->listaRps[] = $xmlRps;
	}

	private function getXmlEnviarLoteRps() {
	    $this->xmlEnviarLoteRps = str_replace("&", "&#38;", $this->xmlEnviarLoteRps);
		$domXml = new \DOMDocument("1.0", "UTF-8");
		$domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        //$domXml = $this->assinaXml($domXml, array("InfRps", "LoteRps"));

        $this->limparListaRps();
        return $domXml->saveXML($domXml->documentElement);

	}

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "RecepcionarLoteRps";

        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        if ($resultado) {
            $nomeArquivoResposta = $prefixoArquivo."resp.xml";
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado);

            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);

            if ($domResult->getElementsByTagName("NumeroLote")->length > 0) {

                $protocolo = $domResult->getElementsByTagName("Protocolo")->item(0)->nodeValue;

                foreach ($this->listaAuxRps as $nota) {
                    Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }

                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());

                return true;
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;

                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                    );
                }
                
                file_put_contents("/tmp/".$nomeArquivoEnvio, $xml);
                return $erros;
            }
            else if($domResult->getElementsByTagName("Erro")->length > 0) {
                 $erros[] = array(
                    "DESCRICAO" => $domResult->getElementsByTagName("Mensagem")->item(0)->nodeValue
                );
                
                file_put_contents("/tmp/".$nomeArquivoEnvio, $xml);
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarLoteRps";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        if ($resultado) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado);
            return $resultado;
        }
        else {
            return false;
        }
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        //$this->RazaoSocialPrestador = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $lote["CDINSCMUNI"]);
        $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
  
        $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);

        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "ISO-8859-1");
            $domResult->loadXML($resultado);
            if ($domResult->getElementsByTagName("ListaNfse")->length > 0) {

                $nodeList = $domResult->getElementsByTagName("InfNfse");
                
                if ($nodeList->length > 0) {
                    foreach ($nodeList as $nota) {
                        $numeroNota = Util::getChildNodeByName($nota, "Numero")->nodeValue;
                        if($this->ambiente == 2) {
                            $numeroNota = str_replace("T", "", $numeroNota);
                        }
                        $numeroNota = str_pad($numeroNota, 9, "0", STR_PAD_LEFT);
                        $codigoVerificacao = Util::getChildNodeByName($nota, "CodigoVerificacao")->nodeValue;
                        $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                        $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;
                        $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue;
                        $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                        /*==================================*/
                        $dataEmissao = Util::getChildNodeByName($nota, "DataEmissao")->nodeValue;
                        $dataEmissao = substr($dataEmissao, 0 , 19);
                        $dataEmissao = date_create_from_format("Y-m-d\TH:i:s", $dataEmissao);
                        $dataEmissao = $dataEmissao->format("d/m/Y");
    
                        $competencia = $nota->getElementsByTagName("Competencia")->item(0)->nodeValue;
                        $competencia = substr($competencia, 0, 10);
                        $competencia = date_create_from_format("Y-m-d", $competencia);
                        $competencia = $competencia->format("d/m/Y");
                        /*==================================*/
                        
                        Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissao, $competencia, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                        Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                        Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    }
    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    return true;
                }
                else {
                    return null;
                }
            }
            else {
                $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                $qtdRps = $lote["QTDERPS"];
                $nrRps = $nrRps - ($qtdRps + 1);
                
                $erros = array();
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                
                if($nodeList->length > 0) {
                    foreach ($nodeList as $erro) {
                        if($erro->firstChild != null) {

                            $codigoErro = Util::getChildNodeByName($erro, "Codigo")->nodeValue;
                            $descricaoErro = Util::getChildNodeByName($erro, "Mensagem")->nodeValue;
                            
                            $numeroRps = explode(".:", $descricaoErro);
                            if(isset($numeroRps[1])) {
                                $numeroRps = str_replace(" ", "", $numeroRps[1]);
                                $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                            }
                            else {
                                return null;
                            }

                            $erros[] = array(
                                "CODIGO" => $codigoErro,
                                "DESCRICAO" => $descricaoErro,
                                "RPS"    => $numeroRps
                            );
                            // E92 e E180 = lote ainda nao foi processado
                            if ($codigoErro == "E92" || $codigoErro == "E180" || $codigoErro == "92" || $codigoErro == "180") {
                                return $erros;
                            }
                        }
                    }
                    
    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    return $erros;
                }
                else {
                    $nota = $domResult->getElementsByTagName("ListaMensagemRetorno")[0];
                    $codigoErro = Util::getChildNodeByName($nota, "Codigo")->nodeValue;
                    $descricaoErro = Util::getChildNodeByName($nota, "Mensagem")->nodeValue;
                    
                    $erros[] = array(
                        "CODIGO" => $codigoErro,
                        "DESCRICAO" => $descricaoErro,
                        "RPS"    => $lote["NRSEQLOTERPS"]
                    );
                    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    return $erros;
                }
            }
        }
        else {
            return false;
        }
    }

    private function gerarXmlCancelarNota($nrnotafisc, $inscricaoJuridica, $inscricaoMunicipal, $codMotivoCancelamento, $cdmunicibge) {
        $nrnotafisc = substr($nrnotafisc, 2, 7);
        if($this->ambiente == 2) {
            $nrnotafisc = 'T'.$nrnotafisc;
        }
        
        $this->xmlCancelarNota = '<CancelarNfseEnvio xmlns="http://www.abrasf.org.br/nfse.xsd">
            <Pedido xmlns="http://www.abrasf.org.br/nfse.xsd">
                <InfPedidoCancelamento
                    Id="NFSE_'.$nrnotafisc.'">
                    <IdentificacaoNfse>
                        <Numero>'.$nrnotafisc.'</Numero>
                        <CpfCnpj>
			        		<Cnpj>'.$inscricaoJuridica.'</Cnpj>
				        </CpfCnpj>
                        <InscricaoMunicipal>'.$inscricaoMunicipal.'</InscricaoMunicipal>
                        <CodigoMunicipio>'.$cdmunicibge.'</CodigoMunicipio>
                    </IdentificacaoNfse>
                    <CodigoCancelamento>'.$codMotivoCancelamento.'</CodigoCancelamento>
                </InfPedidoCancelamento>
            </Pedido>
        </CancelarNfseEnvio>';
    }

    private function getXmlCancelarNota() {
        $this->xmlCancelarNota = str_replace("&", "&#38;", $this->xmlCancelarNota);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNota, LIBXML_NOBLANKS);
        
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        
        if ($domXml === false) {
            throw new \Exception("Erro inesperado!");
        }
        else {
            return $domXml->saveXML($domXml->documentElement);
        }
    }

    public function cancelarNotas($notas, $codMotivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());

        //$this->RazaoSocialPrestador = $notas[0]["NMRAZSOCEMPR"];
        $errosTotais = array();

        foreach ($notas as $nota) {
            $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $nota["NRINSJURIEST"]);
            $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $nota["CDINSCMUNI"]);
            $this->gerarXmlCancelarNota($nota["NRNOTAFISC"], $inscricaoJuridica, $inscricaoMunicipal, $codMotivo, $cdmunicibge);
            $xml = $this->getXmlCancelarNota();
            
            $resultado = $this->cancelarNotaWS($xml, $nota["CDEMPRESA"], $nota["NRINSJURIEST"], $nota["NRNOTAFISC"]);

            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                if ($nodeList->length > 0) {
                    $errosNota = array();
                    foreach ($nodeList as $erro) {
                        $errosNota[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                        );
                    }
                    $errosTotais[$nota["NRLANCTONF"]] = $errosNota;
                }
                else {
                    $motivos = array(
                        1 => "Erro na emissão",
                        2 => "Serviço não prestado",
                        3 => "Erro na assinatura", 
                        4 => "Duplicidade da nota",
                        5 => "Erro no Processamento"
                    );
                    
                    $descMotivo = strtr($codMotivo, $motivos);

                    Util::inserirNotaCancelada($this->connection, "S", "C", $descMotivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
            }
            else {
                return false;
            }
        }
        return (count($errosTotais) > 0) ? $errosTotais : true;
    }

    private function cancelarNotaWS($xml, $cdempresa, $inscricao, $nrnotafisc) {
        $nomeFuncao = "CancelarNfse";
        $prefixoArquivo = $cdempresa.$inscricao.$nrnotafisc."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        
        if ($resultado) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado);
            return $resultado;
        }
        else {
            return false;
        }
    }

   private function enviarParaWebservice($nomeFuncao, $conteudo) {

        $context = stream_context_create(array(
            'ssl' => array(
                'verify_peer' => false,
                'verify_peer_name' => false,
                'allow_self_signed' => false
            )
        ));

        $options = array(
            'soap_version' => SOAP_1_1,
            'stream_context' => $context,
            'exceptions' => true,
            'proxy_host' => '192.168.122.121',
            'trace'      => 1,
            'proxy_port' => 3128
        );
        
        $conteudo = $this->removeAcentos($conteudo);
        $url = self::URL_PRODUCAO;
        
        
        $conteudo = str_replace("<", "&lt;", $conteudo);
        $conteudo = str_replace(">", "&gt;", $conteudo);
        
        $conteudo  ="<ns1:$nomeFuncao><xml>".$conteudo;
        
        $conteudo.="</xml></ns1:$nomeFuncao>";
        $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
        $client = new \SoapClient($url, $options);
        $resultado = $client->$nomeFuncao($soapVar);
        return $resultado->return;
    }
    
    private function removeAcentos($string){
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/", "/ç/", "/Ç/"),explode(" ","a A e E i I o O u U n N c C"),$string);
    }

    private function gerarXmlConsultarLoteRps($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarLoteRpsEnvio xmlns="http://www.abrasf.org.br/nfse.xsd">
            <Prestador>
                <CpfCnpj>
                    <Cnpj>'.$inscricaoPrestador.'</Cnpj>
                </CpfCnpj>
                <InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</InscricaoMunicipal>
            </Prestador>
            <Protocolo>'.$protocolo.'</Protocolo>
        </ConsultarLoteRpsEnvio>';
        return $this->xmlConsultarLoteRps;
    }

     public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();
        

        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];

        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

        $dadosNfct = Util::getDadosNfCt($this->connection, $notafiscal["CDFILIAL"], $notafiscal["NRLANCTONF"]);
        
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;
        
        $paramnfseinscmuni = Util::getParamNFSeInscMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
            
        // $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        $this->SenhaAcesso  = $paramnfseinscmuni["CDSENHAACESSOWS"];
        
        if(is_null($this->SenhaAcesso)) throw new \Exception("A senha de acesso não foi parametrizada.");
        
        $this->NumeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        $this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$this->NumeroLote;
        $this->InscricaoPrestador = str_replace(array(".","-","/"), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array(".","-","/"), "", $notafiscal["CDINSCMUNIEMP"]);

        $this->RazaoSocialPrestador = $notafiscal["NMRAZSOCEMIT"];

        if(!isset($dadosNfct[0])) {
            if($notafiscal["IDNATUOPEREMPR"] == 1) {
                $municipioIncidencia = (int)$notafiscal["CDMUNICIBGEEMP"];
            }
            else {
                $municipioIncidencia = (int)$notafiscal["CDMUNICIBGESERVPRES"];
            }
        }
        else {
            $municipioIncidencia = $dadosNfct[0]["CDMUNICIBGE"];
        }
        
        $this->CodigoPais               = Util::getCodigoBacenPais($this->connection, $notafiscal["CDPAIS"])[0]["CDPAISBACEN"];
        $this->CodigoPaisCliente        = $notafiscal["CDPAISBACENCLIE"];
        $this->CodigoMunicipioPrestador = $notafiscal["CDMUNICIBGEEMP"];
        $this->MunicipioIncidencia      = $municipioIncidencia;

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
            if (empty($notafiscal["CDINSCMUNIEMP"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDNATUOPEREMPR"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Natureza da operação deve ser informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDREGESPTRIBEMP"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Regime especial de tributação deve ser informado."
                );
            }


            $this->RegimeEspecialTributacao = "";
            $this->OptanteSimplesNacional = 2;

            if ($notafiscal["IDOPTSIMPLES"] == "S") {
                $this->RegimeEspecialTributacao = (int)$notafiscal["IDREGESPTRIBEMP"];
                $this->OptanteSimplesNacional = 1;
            }
            
            $this->IncentivoFiscal      = ($notafiscal["IDINCENTFISCAL"] == 'S') ? 1 : 2;
            // $this->IncentivadorCultural = 2;

            $dataAtual = date_create();
            //$this->DataEmissao = $dataAtual->format("Y-m-d\TH:i:s");
            $this->DataEmissao = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");
            
            $this->NumeroRps = Util::inserirNumeroRps($this->connection, $this->NumeroLote, 2, $notafiscal["CDEMPRESA"], $notafiscal["CDSERIERPS"], null, $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

            $this->SerieRps = $notafiscal["CDSERIERPS"];
            $this->IdRps = "RPS_" . $this->SerieRps . "_" . $this->NumeroRps;
            $this->TipoRps = 1;
            $this->StatusRps = 1;

            $this->InscricaoTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSJURCLIE"]);
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA") {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];
            $this->Endereco = substr($notafiscal["DSENDECLIE"], 0, 125);
            if($this->Endereco != null) {
                $numero = explode(",", $notafiscal["DSENDECLIE"]);
                if(isset($numero[1])) {
                    $numero = $numero[1];
                    $numero = (int) $numero;
                }
                else $numero = '.';
            }    
            $numero = 'dhasudha';
            
            $this->Numero = $numero;
            $this->Complemento = substr($notafiscal["DSCOMPENDCLI"], 0, 60);
            $this->Bairro = substr($notafiscal["NMBAIRCLIE"], 0, 60);
            $this->CodigoMunicipioTomador = $notafiscal["CDMUNICIBGECLIE"];
            $this->Uf = $notafiscal["SGESTADOCLIE"];
            $this->Cep = substr(str_replace(array(".","-"), "", $notafiscal["NRCEPCLIE"]), 0, 8);
            $this->Telefone = substr(str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]), 0, 11);
            $this->Email = substr($notafiscal["DSEMAILCLIENFE"], 0, 80);

            $this->ValorServicos = $notafiscal["VRPRODUTOS"];
            $valorLiquido = $notafiscal["VRPRODUTOS"];
            $this->ValorDeducoes = 0.0;
            $this->ValorPis = 0.0;
            $this->ValorCofins = 0.0;
            $this->ValorInss = 0.0;
            $this->ValorIr = 0.0;
            $this->ValorCsll = 0.0;
            $this->IssRetido = 2;
            $this->ValorIss = 0.0;
            $this->ValorIssRetido = 0.0;
            $this->OutrasRetencoes = 0.0;
            $this->DescontoCondicionado = 0.0;
            $this->DescontoIncondicionado = 0.0;
            $this->BaseCalculo = 0.0;

            $impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorPis = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorPis;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCofins = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCofins;
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorInss = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorInss;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorIr = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorIr;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCsll = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCsll;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = 1;
                        $this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        $this->ValorIss       += (float)$imp["VALORIMPOSTO"];
                        $valorLiquido -= $this->ValorIssRetido;
                    }
                    else {
                        $this->ValorIss += (float)$imp["VALORIMPOSTO"];
                        $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                }
            }

            $outrasRetencoes = Util::getOutrasRetencoes($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            if (!empty($outrasRetencoes["VRIMPOSTOIT"]) && $outrasRetencoes["VRIMPOSTOIT"] > 0) {
                $this->OutrasRetencoes = $outrasRetencoes["VALORIMPOSTO"];
                $valorLiquido -= $this->OutrasRetencoes;
            }

            $this->ValorLiquidoNfse = $valorLiquido;

            if (count($impostos) > 0) {
                $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
                $this->Aliquota = 0;
                if (!empty($impostoItem)) {
                    $this->Aliquota = (float)$impostoItem["VRALIQUOTA"];
                }
            }
            else {
                //VALIDACAO
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $cdarvprod = $itemProduto[0]["CDARVPROD"];

            $discriminacaoNota = "";

            foreach ($itemProduto as $linha) {
                $discriminacaoNota .= $linha["DSPRODEMIS"] . "\r\n";
            }

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            //$descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                //$descricaoEncargo .= $item["DESCRICAO"] . "\n";
                if ($item["IDENCARGO"] == "D") {
                    $this->DescontoIncondicionado += (float)$item["VRITEMENCA"];
                }
            }

            /*$descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->Discriminacao = $descricaoProd;
            if (!empty($descricaoEncargo)) {
                $this->Discriminacao .= "|" . $descricaoEncargo;
            }
            if (!empty($descricaoDadosAdic)) {
                $this->Discriminacao .= "|" . $descricaoDadosAdic;
            }*/

            $this->Discriminacao = trim($discriminacaoNota);
            
            //$this->Discriminacao = substr(Util::retiraAcento($this->Discriminacao), 0, 2000);
            $this->Discriminacao = substr($this->Discriminacao, 0, 2000);
            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da classificação do serviço deve ser informado."
                );
            }
            else {
                $this->ItemListaServico = str_replace(array(".", ","), "", $codTribMuni["CDCLASSERVPADR"]);//1.01
                $this->CodigoTributacaoMunicipio = $codTribMuni["CDTRIBPRODPADR"];
                $this->CodigoCnae = $codTribMuni["CDTRIBPRODPADR"];
            }
            $this->FraseSecreta = Util::getFraseSecreta($this->conection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
            $this->adicionarRpsEnviarLote();

            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRps,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $this->NumeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }

        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $this->NumeroLote);
        }
    }

}